/* source_search_flag : tags and macros */
#define MLTAG_VISIBLE_ONLY	((value)(-365153351*2+1))
#define MLTAG_TEXT_ONLY	((value)(-357645954*2+1))
#define MLTAG_CASE_INSENSITIVE	((value)(113105954*2+1))

extern const lookup_info ml_table_source_search_flag[];
#define Val_source_search_flag(data) ml_lookup_from_c (ml_table_source_search_flag, data)
#define Source_search_flag_val(key) ml_lookup_to_c (ml_table_source_search_flag, key)

/* source_smart_home_end_type : tags and macros */
#define MLTAG_DISABLED	((value)(-712388228*2+1))
#define MLTAG_BEFORE	((value)(-860553089*2+1))
#define MLTAG_AFTER	((value)(462462460*2+1))
#define MLTAG_ALWAYS	((value)(-111559985*2+1))

extern const lookup_info ml_table_source_smart_home_end_type[];
#define Val_source_smart_home_end_type(data) ml_lookup_from_c (ml_table_source_smart_home_end_type, data)
#define Source_smart_home_end_type_val(key) ml_lookup_to_c (ml_table_source_smart_home_end_type, key)

/* source_draw_spaces_flags : tags and macros */
#define MLTAG_SPACE	((value)(-11221850*2+1))
#define MLTAG_TAB	((value)(4191797*2+1))
#define MLTAG_NEWLINE	((value)(209349684*2+1))
#define MLTAG_NBSP	((value)(868286929*2+1))
#define MLTAG_LEADING	((value)(154470054*2+1))
#define MLTAG_TEXT	((value)(934974637*2+1))
#define MLTAG_TRAILING	((value)(-833345316*2+1))

extern const lookup_info ml_table_source_draw_spaces_flags[];
#define Val_source_draw_spaces_flags(data) ml_lookup_from_c (ml_table_source_draw_spaces_flags, data)
#define Source_draw_spaces_flags_val(key) ml_lookup_to_c (ml_table_source_draw_spaces_flags, key)

/* source_completion_activation_flags : tags and macros */
#define MLTAG_INTERACTIVE	((value)(311773634*2+1))
#define MLTAG_USER_REQUESTED	((value)(784210714*2+1))

extern const lookup_info ml_table_source_completion_activation_flags[];
#define Val_source_completion_activation_flags(data) ml_lookup_from_c (ml_table_source_completion_activation_flags, data)
#define Source_completion_activation_flags_val(key) ml_lookup_to_c (ml_table_source_completion_activation_flags, key)

